package main

import (
	"bufio"
	"math/rand"
	"regexp"
	"strings"
	"time"
)

type ParsedLink struct {
	Name     string
	PageName string
	Line     string
}

var random *rand.Rand

func init() {
	random = rand.New(rand.NewSource(time.Now().UnixNano()))
}

const letterBytes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

func RandStringBytes(n int) string {
	b := make([]byte, n)
	for i := range b {
		b[i] = letterBytes[rand.Intn(len(letterBytes))]
	}
	return string(b)
}

func ParseLinks(content string) ([]ParsedLink, error) {
	hrefRE := regexp.MustCompile(`(#?\[\[\s*([^\]]+)\s*\]\])`)

	scanner := bufio.NewScanner(strings.NewReader(content))
	scanner.Split(bufio.ScanLines)

	var result []ParsedLink

	for scanner.Scan() {
		line := scanner.Text()

		links := hrefRE.FindAllStringSubmatch(line, -1)

		for _, matches := range links {
			link := matches[0]

			if link[0] == '#' {
				link = strings.TrimPrefix(link, "#[[")
			} else {
				link = strings.TrimPrefix(link, "[[")
			}

			link = strings.TrimSuffix(link, "]]")
			link = strings.TrimSpace(link)
			l := cleanNameURL(link)
			result = append(result, ParsedLink{link, l, line})
		}
	}

	return result, nil
}

func cleanNameURL(name string) string {
	return strings.Replace(name, " ", "_", -1)
}

func cleanTitle(name string) string {
	return strings.Replace(name, "_", " ", -1)
}
